
/**
 * @vue-cdk/tooltip
 * Version: 0.7.2,
 * (c) Christian Kienle, 2020
 * LICENCE: MIT
 * https://github.com/ChristianKienle/cdk
*/
import pluginify from '@vue-cdk/pluginify';
import Popover from '@vue-cdk/popover/src/popover';
import Vue from 'vue';

// @ts-check

/**
 * @template {keyof DocumentEventMap} K
 * @param {K} type
 * @param {HTMLElement | Document} subject
 * @param {(event: DocumentEventMap[K]) => void} handler
 */
var createEventListener = function createEventListener(type, subject, handler) {
  var _isOn = false;
  return {
    isOn: function isOn() {
      return _isOn;
    },
    on: function on() {
      if (_isOn) {
        throw Error('cannot enable a listener that is already enabled');
      }

      _isOn = true;
      subject.addEventListener(type, handler);
      return this;
    },
    off: function off() {
      if (_isOn === false) {
        throw Error('cannot disable a listener that is not enabled');
      }

      _isOn = false;
      subject.removeEventListener(type, handler);
      return this;
    }
  };
};

// @ts-check
/**
 * @param {() => {}} callback
 * @param {{element?: HTMLElement }} options
 */

var createOnEscListener = (function (callback) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var element = options.element || document;
  return createEventListener('keyup', element, function (event) {
    if (event.key === 'Escape') {
      callback();
    }
  });
});

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

//
//
//
var script = {
  name: 'PopoverArrow'
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier
/* server only */
, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
  if (typeof shadowMode !== 'boolean') {
    createInjectorSSR = createInjector;
    createInjector = shadowMode;
    shadowMode = false;
  } // Vue.extend constructor export interop.


  var options = typeof script === 'function' ? script.options : script; // render functions

  if (template && template.render) {
    options.render = template.render;
    options.staticRenderFns = template.staticRenderFns;
    options._compiled = true; // functional template

    if (isFunctionalTemplate) {
      options.functional = true;
    }
  } // scopedId


  if (scopeId) {
    options._scopeId = scopeId;
  }

  var hook;

  if (moduleIdentifier) {
    // server build
    hook = function hook(context) {
      // 2.3 injection
      context = context || // cached call
      this.$vnode && this.$vnode.ssrContext || // stateful
      this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext; // functional
      // 2.2 with runInNewContext: true

      if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
        context = __VUE_SSR_CONTEXT__;
      } // inject component styles


      if (style) {
        style.call(this, createInjectorSSR(context));
      } // register component module identifier for async chunk inference


      if (context && context._registeredComponents) {
        context._registeredComponents.add(moduleIdentifier);
      }
    }; // used by ssr in case component is cached and beforeCreate
    // never gets called


    options._ssrRegister = hook;
  } else if (style) {
    hook = shadowMode ? function (context) {
      style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
    } : function (context) {
      style.call(this, createInjector(context));
    };
  }

  if (hook) {
    if (options.functional) {
      // register for functional component in vue file
      var originalRender = options.render;

      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      // inject component registration as beforeCreate hook
      var existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }

  return script;
}

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    attrs: {
      "data-popper-arrow": "true"
    }
  });
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* module identifier */

var __vue_module_identifier__ = undefined;
/* functional template */

var __vue_is_functional_template__ = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__ = normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, undefined, undefined, undefined);

// @ts-check

/** @typedef {string | null | undefined} Value */

/**
 * @param {any} cls
 * @returns {boolean}
 */
var invalidClasses = function invalidClasses(cls) {
  return typeof cls === 'string' && cls.length > 0;
};
/**
 * @param {Array.<Value>} classes
 * @returns {Array.<string> | null}
 */


var _classes = function _classes(classes) {
  // Filter out null, undefined and ""
  var onlyClasses = classes.filter(invalidClasses);

  if (onlyClasses.length === 0) {
    return null;
  } // @ts-ignore


  return onlyClasses;
};

//
var script$1 = {
  name: 'PopoverContent',
  components: {
    CPopoverArrow: __vue_component__
  },
  inject: {
    $_popover: {
      default: {
        theme: null,
        withArrow: false,
        arrowClasses: []
      }
    }
  },
  computed: {
    injectedArrowClasses: function injectedArrowClasses() {
      return this.$_popover.arrowClasses;
    },
    arrowClasses: function arrowClasses() {
      var theme = this.theme,
          injectedArrowClasses = this.injectedArrowClasses;
      return _classes([injectedArrowClasses, theme ? 'vcdk-popover--arrow' : null]);
    },
    theme: function theme() {
      return this.$_popover.theme;
    },
    classes: function classes() {
      var theme = this.theme,
          withArrow = this.withArrow;
      return _classes(['vcdk-popover-body', theme ? "vcdk-popover-theme-".concat(this.theme) : null, !withArrow ? 'vcdk-popover--no-arrow' : null]);
    },
    withArrow: function withArrow() {
      return this.$_popover.withArrow;
    }
  }
};

/* script */
var __vue_script__$1 = script$1;
/* template */

var __vue_render__$1 = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    class: _vm.classes
  }, [_vm._t("default"), _vm._v(" "), _vm.withArrow ? _vm._t("arrow", [_c('CPopoverArrow', {
    class: _vm.arrowClasses
  })]) : _vm._e()], 2);
};

var __vue_staticRenderFns__$1 = [];
/* style */

var __vue_inject_styles__$1 = undefined;
/* scoped */

var __vue_scope_id__$1 = undefined;
/* module identifier */

var __vue_module_identifier__$1 = undefined;
/* functional template */

var __vue_is_functional_template__$1 = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__$1 = normalizeComponent({
  render: __vue_render__$1,
  staticRenderFns: __vue_staticRenderFns__$1
}, __vue_inject_styles__$1, __vue_script__$1, __vue_scope_id__$1, __vue_is_functional_template__$1, __vue_module_identifier__$1, false, undefined, undefined, undefined);

var script$2 = {
  name: 'TooltipContent',
  functional: true,
  render: function render(h, context) {
    var scopedSlots = {
      default: context.scopedSlots.default
    };

    var attrs = _objectSpread2({}, context.data.attrs, {
      role: 'tooltip'
    });

    var data = _objectSpread2({}, context.data, {
      scopedSlots: scopedSlots,
      attrs: attrs
    });

    return h(__vue_component__$1, data, [context.children]);
  }
};

/* script */
var __vue_script__$2 = script$2;
/* template */

/* style */

var __vue_inject_styles__$2 = undefined;
/* scoped */

var __vue_scope_id__$2 = undefined;
/* module identifier */

var __vue_module_identifier__$2 = undefined;
/* functional template */

var __vue_is_functional_template__$2 = undefined;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__$2 = normalizeComponent({}, __vue_inject_styles__$2, __vue_script__$2, __vue_scope_id__$2, __vue_is_functional_template__$2, __vue_module_identifier__$2, false, undefined, undefined, undefined);

var script$3 = {
  name: 'Tooltip',
  components: {
    Popover: Popover
  },
  props: {
    noArrow: {
      type: Boolean,
      default: false
    },
    visible: {
      type: Boolean,
      default: null
    },
    theme: {
      type: String,
      default: null
    },
    text: {
      type: String,
      default: null
    },
    placement: {
      type: String,
      default: 'top'
    }
  },
  computed: {
    popoverTheme: function popoverTheme() {
      var theme = this.theme;

      if (theme == null) {
        return null;
      }

      return "tooltip-".concat(this.theme);
    }
  },
  watch: {
    visible: function visible(_visible) {
      var $_popover = this.$_popover;

      if ($_popover == null) {
        return;
      }

      if (_visible) {
        $_popover.show();
      } else {
        $_popover.hide();
      }
    },
    popoverTheme: function popoverTheme(_popoverTheme) {
      var $_popover = this.$_popover;

      if ($_popover == null) {
        return;
      }

      $_popover.theme = _popoverTheme;
    }
  },
  beforeUpdate: function beforeUpdate() {
    if (this.$el == null) {
      return;
    }

    this.turnListenersOff();
  },
  updated: function updated() {
    var $el = this.$el;

    if ($el == null) {
      return;
    }

    this.setupListeners();
    this.turnListenersOn();
  },
  destroy: function destroy() {
    this.$_popover.$destory();
    this.turnListenersOff();
  },
  mounted: function mounted() {
    var _this = this;

    var target = function target() {
      return _this.$el;
    };

    var theme = this.popoverTheme;
    this.$_escListener = createOnEscListener(function () {
      return _this.handleESC();
    });
    var tooltipVM = this;
    this.$_popover = new Vue({
      components: {
        Popover: Popover
      },
      data: function data() {
        return {
          theme: theme,
          noArrow: tooltipVM.noArrow,
          visible: tooltipVM.visible,
          placement: tooltipVM.placement
        };
      },
      methods: {
        show: function show() {
          this.$refs.popover.show();
          this.visible = true;
        },
        hide: function hide() {
          this.$refs.popover.hide();
          this.visible = false;
        }
      },
      render: function render(h) {
        var _this2 = this;

        var props = {
          target: target,
          withArrow: !this.noArrow,
          theme: this.theme,
          placement: this.placement
        };
        var on = {
          'update:visible': function updateVisible(newVisible) {
            if (_this2.visible != null) {
              return;
            }

            _this2.visible = newVisible;
          }
        };
        var scopedSlots = {
          default: function _default(popoverProps) {
            var text = tooltipVM.text,
                $scopedSlots = tooltipVM.$scopedSlots;

            if (text != null) {
              return h(__vue_component__$2, {}, [text]);
            }

            return $scopedSlots.content(popoverProps);
          }
        };
        return h(Popover, {
          ref: 'popover',
          on: on,
          parent: tooltipVM,
          props: props,
          scopedSlots: scopedSlots
        });
      }
    }).$mount();
    this.$forceUpdate();
  },
  methods: {
    setupListeners: function setupListeners() {
      var _this3 = this;

      var visible = this.visible;

      this.$_mouseEnterListener = createEventListener('mouseenter', this.$el, function () {
        _this3.handleMouseEnter();
      });
      this.$_mouseLeaveListener = createEventListener('mouseleave', this.$el, function () {
        _this3.handleMouseLeave();
      });
      this.$_blurListener = createEventListener('blur', this.$el, function () {
        return _this3.handleBlur();
      });
      this.$_focusListener = createEventListener('focus', this.$el, function () {
        return _this3.handleFocus();
      });
    },
    turnListenersOff: function turnListenersOff() {
      this.$_mouseEnterListener != null && this.$_mouseEnterListener.isOn() && this.$_mouseEnterListener.off();
      this.$_mouseLeaveListener != null && this.$_mouseLeaveListener.isOn() && this.$_mouseLeaveListener.off();
      this.$_blurListener != null && this.$_blurListener.isOn() && this.$_blurListener.off();
      this.$_focusListener != null && this.$_focusListener.isOn() && this.$_focusListener.off();
    },
    turnListenersOn: function turnListenersOn() {
      var visible = this.visible;

      this.$_mouseEnterListener.isOn() === false && this.$_mouseEnterListener.on();
      this.$_mouseLeaveListener.isOn() === false && this.$_mouseLeaveListener.on();
      this.$_blurListener.isOn() === false && this.$_blurListener.on();
      this.$_focusListener.isOn() === false && this.$_focusListener.on();
    },
    handleMouseEnter: function handleMouseEnter() {
      this.$_popover.show();
      this.$_escListener.isOn() === false && this.$_escListener.on();
    },
    handleMouseLeave: function handleMouseLeave() {
      this.$_popover.hide();
      this.$_escListener.isOn() && this.$_escListener.off();
    },
    handleBlur: function handleBlur() {
      this.$_popover.hide();
      this.$_escListener.isOn() && this.$_escListener.off();
    },
    handleFocus: function handleFocus() {
      this.$_popover.show();
      this.$_escListener.isOn() === false && this.$_escListener.on();
    },
    handleESC: function handleESC() {
      this.$_popover.hide();
      this.$_escListener.isOn() && this.$_escListener.off();
    }
  },
  render: function render() {
    return this.$slots.default;
  }
};

/* script */
var __vue_script__$3 = script$3;
/* template */

/* style */

var __vue_inject_styles__$3 = undefined;
/* scoped */

var __vue_scope_id__$3 = undefined;
/* module identifier */

var __vue_module_identifier__$3 = undefined;
/* functional template */

var __vue_is_functional_template__$3 = undefined;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__$3 = normalizeComponent({}, __vue_inject_styles__$3, __vue_script__$3, __vue_scope_id__$3, __vue_is_functional_template__$3, __vue_module_identifier__$3, false, undefined, undefined, undefined);

var index = pluginify(__vue_component__$3, __vue_component__$2);

export default index;
